<?php

namespace Drupal\raisedonors\BatchProcess;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Link;
use Drupal\Core\Url;

/**
 * Batch processing for donor synchronization.
 */
class RaisedonorsSyncBatch {

  use StringTranslationTrait;
  
  /**
   * Initialize the batch process.
   * 
   * @param array &$context
   *   The batch context.
   */
  public static function batchStart(array &$context) {
    $context['results']['processed'] = 0;
    $context['results']['errors'] = 0;
    $context['results']['created'] = 0;
    $context['results']['updated'] = 0;
    $context['results']['skipped'] = 0;
    $context['results']['started'] = time();
    
    // Initialize sandbox information for batching through large datasets.
    $context['sandbox']['current_page'] = 0;
    $context['sandbox']['total_pages'] = 0;
    $context['sandbox']['total_donors'] = 0;
    
    // Get service for API communication.
    $api_service = \Drupal::service('raisedonors.api_service');
    
    // Test connection first.
    if (!$api_service->testConnection()) {
      $context['results']['error'] = t('Could not connect to RaiseDonors API. Please check your API settings.');
      // Force batch completion.
      $context['finished'] = 1;
      return;
    }
    
    // Fetch first page to get total count.
    try {
      $response = $api_service->makeRequest('GET', '/Donor/list', [
        'Take' => 100,
        'Skip' => 0,
        'SortBy' => 'Id',
        'Descending' => 'true',
      ]);
      
      if (!empty($response) && isset($response['total'])) {
        $total_donors = $response['total'];
        $context['sandbox']['total_donors'] = $total_donors;
        $context['sandbox']['total_pages'] = ceil($total_donors / 100);
        
        $context['message'] = t('Beginning sync of @total donors...', [
          '@total' => $total_donors,
        ]);
      }
      else {
        $context['results']['error'] = t('Could not retrieve donor count from RaiseDonors API.');
        $context['finished'] = 1;
      }
    }
    catch (\Exception $e) {
      $context['results']['error'] = t('Error initializing sync: @error', [
        '@error' => $e->getMessage(),
      ]);
      $context['finished'] = 1;
    }
  }
  
  /**
   * Process a batch of donors.
   * 
   * @param array &$context
   *   The batch context.
   */
  public static function batchProcess(array &$context) {
    // Check if we've been directed to stop.
    if (isset($context['finished']) && $context['finished'] >= 1) {
      return;
    }
    
    // Get required services.
    $api_service = \Drupal::service('raisedonors.api_service');
    $config = \Drupal::config('raisedonors.settings');
    $logger = \Drupal::logger('raisedonors');
    
    // If we're in test mode, log and skip actual processing.
    if ($config->get('mode') === 'test') {
      $context['results']['skipped'] = $context['sandbox']['total_donors'];
      $context['message'] = t('Test mode enabled. Skipping donor sync.');
      $context['finished'] = 1;
      return;
    }
    
    // Process the current page.
    $current_page = $context['sandbox']['current_page'];
    $skip = $current_page * 100;
    
    try {
      $response = $api_service->makeRequest('GET', '/Donor/list', [
        'Take' => 100,
        'Skip' => $skip,
        'SortBy' => 'Id',
        'Descending' => 'true',
      ]);
      
      if (empty($response) || !isset($response['donors']) || empty($response['donors'])) {
        // No data found, mark as finished.
        $context['finished'] = 1;
        return;
      }
      
      // Process each donor.
      foreach ($response['donors'] as $donor) {
        try {
          // Check if we already have a user for this donor.
          $user_service = \Drupal::service('raisedonors.user_service');
          $existing_user = $user_service->findUserByDonorId($donor['id']);
          
          if ($existing_user) {
            // Update existing user.
            $user_service->updateUser($existing_user, $donor);
            $context['results']['updated']++;
          }
          else {
            // Create new user.
            $new_user = $user_service->createUser($donor);
            if ($new_user) {
              $context['results']['created']++;
            }
            else {
              $context['results']['errors']++;
              $logger->error('Failed to create user for donor @id during batch sync', [
                '@id' => $donor['id'] ?? 'unknown',
              ]);
            }
          }
          
          $context['results']['processed']++;
        }
        catch (\Exception $e) {
          $context['results']['errors']++;
          $logger->error('Error processing donor @id during batch sync: @error', [
            '@id' => $donor['id'] ?? 'unknown',
            '@error' => $e->getMessage(),
          ]);
        }
      }
      
      // Update progress.
      $context['sandbox']['current_page']++;
      if ($context['sandbox']['total_pages'] > 0) {
        $context['finished'] = $context['sandbox']['current_page'] / $context['sandbox']['total_pages'];
      }
      else {
        $context['finished'] = 1;
      }
      
      // Show progress message.
      $processed = $context['results']['processed'];
      $total = $context['sandbox']['total_donors'];
      $percent = ($processed / $total) * 100;
      
      $context['message'] = t('Processed @processed of @total donors (@percent%)', [
        '@processed' => $processed,
        '@total' => $total,
        '@percent' => number_format($percent, 1),
      ]);
    }
    catch (\Exception $e) {
      $context['results']['errors']++;
      $logger->error('Batch sync error on page @page: @error', [
        '@page' => $current_page,
        '@error' => $e->getMessage(),
      ]);
      
      // Continue to next page despite error.
      $context['sandbox']['current_page']++;
      if ($context['sandbox']['total_pages'] > 0) {
        $context['finished'] = $context['sandbox']['current_page'] / $context['sandbox']['total_pages'];
      }
      else {
        $context['finished'] = 1;
      }
    }
  }
  
  /**
   * Final batch operation to clean up.
   * 
   * @param array &$context
   *   The batch context.
   */
  public static function batchFinish(array &$context) {
    // Update the last sync timestamp in the configuration.
    $config_factory = \Drupal::configFactory();
    $config = $config_factory->getEditable('raisedonors.settings');
    $config->set('last_sync', time());
    $config->save();
    
    // Set final results.
    $context['results']['ended'] = time();
    $context['results']['duration'] = $context['results']['ended'] - $context['results']['started'];
    
    $context['message'] = t('Finalizing donor synchronization...');
  }
  
  /**
   * Batch 'finished' callback.
   * 
   * @param bool $success
   *   Whether the batch process was successful.
   * @param array $results
   *   The batch results.
   * @param array $operations
   *   The operations processed.
   */
  public static function batchFinished($success, array $results, array $operations) {
    $messenger = \Drupal::messenger();
    
    if ($success) {
      // Display success message.
      if (isset($results['error'])) {
        $messenger->addError($results['error']);
      }
      else {
        $time_taken = \Drupal::service('date.formatter')->formatInterval($results['duration']);
        
        $messenger->addStatus(t('Donor synchronization completed in @time.', [
          '@time' => $time_taken,
        ]));
        
        $messenger->addStatus(t('Processed @total donors: @created created, @updated updated, @errors errors.', [
          '@total' => $results['processed'],
          '@created' => $results['created'],
          '@updated' => $results['updated'],
          '@errors' => $results['errors'],
        ]));
      }
    }
    else {
      // An error occurred.
      $messenger->addError(t('An error occurred during donor synchronization.'));
    }
  }
}
