<?php

namespace Drupal\raisedonors\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\raisedonors\Service\RaisedonorsApiService;
use Drupal\raisedonors\Service\RaisedonorsDrupalUserService;

/**
 * Handles RaiseDonors webhook callbacks.
 */
class RaisedonorsWebhookController extends ControllerBase {

  /**
   * The RaiseDonors API service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsApiService
   */
  protected $apiService;

  /**
   * The RaiseDonors user service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsDrupalUserService
   */
  protected $userService;

  /**
   * Constructs a new RaisedonorsWebhookController object.
   *
   * @param \Drupal\raisedonors\Service\RaisedonorsApiService $api_service
   *   The RaiseDonors API service.
   * @param \Drupal\raisedonors\Service\RaisedonorsDrupalUserService $user_service
   *   The RaiseDonors user service.
   */
  public function __construct(
    RaisedonorsApiService $api_service,
    RaisedonorsDrupalUserService $user_service
  ) {
    $this->apiService = $api_service;
    $this->userService = $user_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('raisedonors.api_service'),
      $container->get('raisedonors.user_service')
    );
  }

  /**
   * Validates webhook security token.
   *
   * @param array $data
   *   The webhook payload data.
   *
   * @return bool
   *   TRUE if the token is valid, FALSE otherwise.
   */
  protected function validateWebhookToken(array $data): bool {
    // Get the configured token from settings.
    $config = $this->config('raisedonors.settings');
    $configured_token = $config->get('webhook_token');
    
    // If no token is configured, log a warning but allow the request
    // (for backward compatibility).
    if (empty($configured_token)) {
      $this->getLogger('raisedonors')->warning('No webhook security token configured. Consider setting one for improved security.');
      return TRUE;
    }
    
    // Check if the security token was included in the payload.
    if (!isset($data['SecurityToken']) || empty($data['SecurityToken'])) {
      $this->getLogger('raisedonors')->error('Webhook request missing security token');
      return FALSE;
    }
    
    // Validate that the tokens match using a timing-safe comparison.
    return hash_equals($configured_token, $data['SecurityToken']);
  }

  /**
   * Handles donor created webhook.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function donorCreated(Request $request): JsonResponse {
    $content = $request->getContent();
    $data = json_decode($content, TRUE);
    
    if (empty($data) || !isset($data['Id'])) {
      return new JsonResponse(['error' => 'Invalid webhook data'], 400);
    }
    
    // Validate security token.
    if (!$this->validateWebhookToken($data)) {
      $this->getLogger('raisedonors')->error('Webhook request with invalid security token received');
      return new JsonResponse(['error' => 'Unauthorized - Invalid security token'], 403);
    }
    
    try {
      // Check if we're in test mode.
      $config = $this->config('raisedonors.settings');
      if ($config->get('mode') === 'test') {
        return new JsonResponse(['status' => 'skipped', 'message' => 'Test mode enabled']);
      }
      
      // Create the user using webhook data.
      $user = $this->userService->createUser($data);
      
      if ($user) {
        return new JsonResponse(['status' => 'success', 'message' => 'User created']);
      }
      else {
        return new JsonResponse(['error' => 'Failed to create user'], 500);
      }
    }
    catch (\Exception $e) {
      $this->getLogger('raisedonors')->error('Webhook error: @error', ['@error' => $e->getMessage()]);
      return new JsonResponse(['error' => 'Internal server error'], 500);
    }
  }

  /**
   * Handles donor updated webhook.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function donorUpdated(Request $request): JsonResponse {
    $content = $request->getContent();
    $data = json_decode($content, TRUE);
    
    if (empty($data) || !isset($data['Id'])) {
      return new JsonResponse(['error' => 'Invalid webhook data'], 400);
    }
    
    // Validate security token.
    if (!$this->validateWebhookToken($data)) {
      $this->getLogger('raisedonors')->error('Webhook request with invalid security token received');
      return new JsonResponse(['error' => 'Unauthorized - Invalid security token'], 403);
    }
    
    try {
      // Check if we're in test mode.
      $config = $this->config('raisedonors.settings');
      if ($config->get('mode') === 'test') {
        return new JsonResponse(['status' => 'skipped', 'message' => 'Test mode enabled']);
      }
      
      // Find the user.
      $user = $this->userService->findUserByDonorId($data['Id']);
      
      if ($user) {
        // Update the user with webhook data.
        $this->userService->updateUser($user, $data);
        return new JsonResponse(['status' => 'success', 'message' => 'User updated']);
      }
      else {
        // Create the user if not found, using webhook data.
        $user = $this->userService->createUser($data);
        return new JsonResponse(['status' => 'success', 'message' => 'User created']);
      }
    }
    catch (\Exception $e) {
      $this->getLogger('raisedonors')->error('Webhook error: @error', ['@error' => $e->getMessage()]);
      return new JsonResponse(['error' => 'Internal server error'], 500);
    }
  }

  /**
   * Handles donor deleted webhook.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response.
   */
  public function donorDeleted(Request $request): JsonResponse {
    $content = $request->getContent();
    $data = json_decode($content, TRUE);
    
    if (empty($data) || !isset($data['Id'])) {
      return new JsonResponse(['error' => 'Invalid webhook data'], 400);
    }
    
    // Validate security token.
    if (!$this->validateWebhookToken($data)) {
      $this->getLogger('raisedonors')->error('Webhook request with invalid security token received');
      return new JsonResponse(['error' => 'Unauthorized - Invalid security token'], 403);
    }
    
    try {
      // Check if we're in test mode.
      $config = $this->config('raisedonors.settings');
      if ($config->get('mode') === 'test') {
        return new JsonResponse(['status' => 'skipped', 'message' => 'Test mode enabled']);
      }
      
      // Find the user by donor ID.
      $user = $this->userService->findUserByDonorId($data['Id']);
      
      if ($user) {
        // Delete the user.
        $user->delete();
        
        $this->getLogger('raisedonors')->info('Deleted Drupal user for donor @id', ['@id' => $data['Id']]);
        
        return new JsonResponse(['status' => 'success', 'message' => 'User deleted']);
      }
      else {
        // No user found with this donor ID.
        return new JsonResponse(['status' => 'not_found', 'message' => 'No user found for this donor ID']);
      }
    }
    catch (\Exception $e) {
      $this->getLogger('raisedonors')->error('Webhook error during donor deletion: @error', ['@error' => $e->getMessage()]);
      return new JsonResponse(['error' => 'Internal server error'], 500);
    }
  }
}
