<?php

namespace Drupal\raisedonors\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\raisedonors\Service\RaisedonorsApiService;
use Drupal\key\KeyRepositoryInterface;

/**
 * Configure RaiseDonors settings for this site.
 */
class RaisedonorsSettingsForm extends ConfigFormBase {

  /**
   * The RaiseDonors API service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsApiService
   */
  protected $apiService;

  /**
   * The key repository.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * Constructs a new RaisedonorsSettingsForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Config\TypedConfigManagerInterface $typed_config
   *   The typed config manager service.
   * @param \Drupal\raisedonors\Service\RaisedonorsApiService $api_service
   *   The RaiseDonors API service.
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key repository service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    TypedConfigManagerInterface $typed_config,
    RaisedonorsApiService $api_service,
    KeyRepositoryInterface $key_repository
  ) {
    // Call parent constructor with config factory and typed config manager
    parent::__construct($config_factory, $typed_config);
    
    $this->apiService = $api_service;
    $this->keyRepository = $key_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('raisedonors.api_service'),
      $container->get('key.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['raisedonors.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'raisedonors_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('raisedonors.settings');
    $mode = $config->get('mode') ?: 'test';

    // Mode banner
    if ($mode === 'live') {
        $form['mode_banner'] = [
            '#type' => 'container',
            '#attributes' => [
                'class' => ['messages', 'messages--status'],
            ],
            '#markup' => $this->t('Operating in live mode. Donors from RaiseDonors will sync based on synchronization settings defined below.'),
            '#weight' => -100,
        ];
    } elseif ($mode === 'test') {
        $form['mode_banner'] = [
            '#type' => 'container',
            '#attributes' => [
                'class' => ['messages', 'messages--warning'],
            ],
            '#markup' => $this->t('Operating in test mode. Donors from RaiseDonors will not sync.'),
            '#weight' => -100,
        ];
    }

    // Mode selection (Live or Test).
    $form['mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Mode'),
      '#options' => [
        'live' => $this->t('Live'),
        'test' => $this->t('Test'),
      ],
      '#default_value' => $mode,
      '#description' => $this->t('In Test mode, no new users will be created in the database.'),
    ];

    // API Key selection.
    $form['api_key'] = [
      '#type' => 'key_select',
      '#title' => $this->t('RaiseDonors API Key'),
      '#description' => $this->t('Select the key containing your RaiseDonors API credentials.'),
      '#default_value' => $config->get('api_key'),
      '#required' => TRUE,
      '#key_filters' => ['type' => 'authentication'],
    ];

    // Test API Connection button
    $form['test_connection'] = [
      '#type' => 'button',
      '#value' => $this->t('Test API Connection'),
      '#ajax' => [
        'callback' => '::testApiConnection',
        'wrapper' => 'connection-result',
        'event' => 'click',
      ],
      '#prefix' => '<div class="form-item">',
      '#suffix' => '</div>',
    ];

    // Container for API test result message.
    $form['connection_result'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'connection-result'],
    ];

    // Webhook settings
    $form['webhook_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Webhook Settings'),
      '#open' => TRUE,
    ];

    $form['webhook_settings']['webhook_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Webhook Security Token'),
      '#description' => $this->t('Enter the security token configured in RaiseDonors for webhooks. This token will be used to verify incoming webhook requests.'),
      '#default_value' => $config->get('webhook_token'),
      '#required' => FALSE,
      '#attributes' => [
        'autocomplete' => 'off',
      ],
    ];

    // Sync settings section
    $form['sync_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Synchronization Settings'),
      '#open' => TRUE,
    ];

    $form['sync_settings']['sync_frequency'] = [
      '#type' => 'select',
      '#title' => $this->t('Automatic Sync Frequency'),
      '#options' => [
        'never' => $this->t('Never (manual only)'),
        'daily' => $this->t('Daily'),
        'weekly' => $this->t('Weekly'),
        'monthly' => $this->t('Monthly'),
      ],
      '#default_value' => $config->get('sync_frequency') ?: 'never',
      '#description' => $this->t('How often should donors be automatically synchronized from RaiseDonors?'),
    ];

    $form['sync_settings']['last_sync'] = [
      '#type' => 'item',
      '#title' => $this->t('Last Synchronization'),
      '#markup' => $this->formatLastSyncInfo($config->get('last_sync')),
      '#description' => $this->t('The last time a full donor synchronization was performed.'),
    ];
    
    // Manual sync button
    $form['sync_settings']['manual_sync'] = [
      '#type' => 'submit',
      '#value' => $this->t('Sync Donors Now'),
      '#submit' => ['::syncDonorsSubmit'],
      '#disabled' => empty($config->get('api_key')),
      '#description' => $this->t('Start a full donor synchronization from RaiseDonors.'),
    ];

    // Sync process help details (accordion)
    $form['sync_settings']['sync_help'] = [
      '#type' => 'details',
      '#title' => $this->t('What happens during synchronization'),
      '#description' => $this->t('Learn more about the donor synchronization process.'),
      '#weight' => 10,
      '#open' => FALSE, // Closed by default
      'content' => [
        '#markup' => $this->t('<ul>
<li>Retrieves all donor records from the RaiseDonors platform</li>
<li>For each donor:
  <ul>
    <li>Checks if a Drupal user already exists based on donor ID</li>
    <li>If user exists, updates existing user information</li>
    <li>If no user exists, creates a new Drupal user</li>
  </ul>
</li>
<li>Updates user fields including:
  <ul>
    <li>Name (first, middle, last, suffix)</li>
    <li>Email address</li>
    <li>Phone number</li>
    <li>Postal address</li>
  </ul>
</li>
<li>Automatically assigns "donor" role to imported/updated users</li>
<li>Logs synchronization results and any errors</li>
</ul>
<strong>Important:</strong>
<p>Synchronization may take several minutes depending on the number of donors in your RaiseDonors account. Do not close the browser during the process.</p>'),
      ],
    ];

    // Uninstall behavior settings
    $form['uninstall_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Uninstallation Settings'),
      '#open' => TRUE,
    ];

    $form['uninstall_settings']['uninstall_users'] = [
      '#type' => 'radios',
      '#title' => $this->t('User Handling During Uninstallation'),
      '#options' => [
        'keep' => $this->t('Keep all users'),
        'disable' => $this->t('Disable users (mark as blocked)'),
        'delete' => $this->t('Delete users created by RaiseDonors'),
      ],
      '#default_value' => $config->get('uninstall_users') ?: 'keep',
      '#description' => $this->t('Determine what happens to users created by RaiseDonors when the module is uninstalled. Warning: Deleting users will remove all their content and cannot be undone.'),
    ];

    // RaiseDonors Login button
    $form['raisedonors_login'] = [
      '#type' => 'link',
      '#title' => $this->t('RaiseDonors Login'),
      '#url' => Url::fromUri('https://app.raisedonors.com/account/login'),
      '#attributes' => [
        'class' => [
          'button', 
          'button-action', 
          'button--primary',
          'raisedonors-login-button'
        ],
        'target' => '_blank',
      ],
      '#weight' => 9999, // Ensure it's at the very bottom
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Formats the last sync timestamp into a human-readable string.
   *
   * @param int|null $timestamp
   *   The timestamp of the last sync.
   *
   * @return string
   *   A formatted string with sync information.
   */
  protected function formatLastSyncInfo($timestamp) {
    if (empty($timestamp)) {
      return $this->t('Never');
    }
    
    $formatted_date = $this->dateFormatter()->format($timestamp, 'medium');
    $time_ago = $this->dateFormatter()->formatDiff($timestamp, time(), ['granularity' => 2]);
    
    return $this->t('@date (@time_ago)', [
      '@date' => $formatted_date,
      '@time_ago' => $time_ago,
    ]);
  }

  /**
   * AJAX callback to test the API connection.
   */
  public function testApiConnection(array &$form, FormStateInterface $form_state): array {
    $api_key_id = $form_state->getValue('api_key');
    
    // Prepare result container.
    $element = [
      '#type' => 'container',
      '#attributes' => ['id' => 'connection-result'],
    ];
    
    if (!empty($api_key_id)) {
      // Set the values temporarily for testing.
      $this->apiService->setApiKeyId($api_key_id);
      
      // Test the connection.
      if ($this->apiService->testConnection()) {
        $element['result'] = [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#value' => $this->t('Successfully connected to RaiseDonors via the supplied API Key!'),
          '#attributes' => [
            'class' => ['messages', 'messages--status'],
          ],
        ];
      }
      else {
        $element['result'] = [
          '#type' => 'html_tag',
          '#tag' => 'div',
          '#value' => $this->t('Cannot connect to RaiseDonors via the supplied API key. Please check your key and try again.'),
          '#attributes' => [
            'class' => ['messages', 'messages--error'],
          ],
        ];
      }
    }
    else {
      $element['result'] = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $this->t('Please provide an API Key before testing.'),
        '#attributes' => [
          'class' => ['messages', 'messages--warning'],
        ],
      ];
    }
    
    return $element;
  }

  /**
   * Submit handler for the manual sync button.
   */
  public function syncDonorsSubmit(array &$form, FormStateInterface $form_state) {
    // Launch the batch process for syncing donors.
    $batch = [
      'title' => $this->t('Synchronizing donors from RaiseDonors'),
      'operations' => [
        ['\Drupal\raisedonors\BatchProcess\RaisedonorsSyncBatch::batchStart', []],
        ['\Drupal\raisedonors\BatchProcess\RaisedonorsSyncBatch::batchProcess', []],
        ['\Drupal\raisedonors\BatchProcess\RaisedonorsSyncBatch::batchFinish', []],
      ],
      'finished' => '\Drupal\raisedonors\BatchProcess\RaisedonorsSyncBatch::batchFinished',
      'progressive' => TRUE,
    ];
    
    batch_set($batch);
    
    // Update the last sync timestamp.
    $this->config('raisedonors.settings')
      ->set('last_sync', time())
      ->save();
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->config('raisedonors.settings')
      ->set('mode', $form_state->getValue('mode'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('webhook_token', $form_state->getValue('webhook_token'))
      ->set('sync_frequency', $form_state->getValue(['sync_settings', 'sync_frequency']))
      ->set('uninstall_users', $form_state->getValue(['uninstall_settings', 'uninstall_users']))
      ->save();
    
    parent::submitForm($form, $form_state);
  }
}
