<?php

namespace Drupal\raisedonors\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\raisedonors\Service\RaisedonorsApiService;
use Drupal\raisedonors\Service\RaisedonorsDrupalUserService;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Processes RaiseDonors donor synchronization tasks.
 *
 * @QueueWorker(
 *   id = "raisedonors_donor_sync",
 *   title = @Translation("RaiseDonors Donor Synchronization"),
 *   cron = {"time" = 60}
 * )
 */
class RaisedonorsDonorSyncQueueWorker extends QueueWorkerBase implements ContainerFactoryPluginInterface {

  /**
   * The RaiseDonors API service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsApiService
   */
  protected $apiService;

  /**
   * The RaiseDonors user service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsDrupalUserService
   */
  protected $userService;

  /**
   * The logger channel.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new RaisedonorsDonorSyncQueueWorker object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\raisedonors\Service\RaisedonorsApiService $api_service
   *   The RaiseDonors API service.
   * @param \Drupal\raisedonors\Service\RaisedonorsDrupalUserService $user_service
   *   The RaiseDonors user service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    RaisedonorsApiService $api_service,
    RaisedonorsDrupalUserService $user_service,
    LoggerInterface $logger
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->apiService = $api_service;
    $this->userService = $user_service;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('raisedonors.api_service'),
      $container->get('raisedonors.user_service'),
      $container->get('logger.channel.raisedonors')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    // Skip if in test mode.
    $config = \Drupal::config('raisedonors.settings');
    if ($config->get('mode') === 'test') {
      $this->logger->notice('Skipping queued donor sync because RaiseDonors module is in test mode.');
      return;
    }
    
    try {
      // Get donors for this page.
      $response = $this->apiService->makeRequest('GET', '/Donor/list', [
        'Take' => $data['take'],
        'Skip' => $data['skip'],
        'SortBy' => 'Id',
        'Descending' => 'true',
      ]);
      
      if (empty($response) || !isset($response['donors']) || empty($response['donors'])) {
        $this->logger->notice('No donors found for page @page during scheduled sync.', [
          '@page' => $data['page'],
        ]);
        return;
      }
      
      $created = 0;
      $updated = 0;
      $errors = 0;
      
      // Process each donor.
      foreach ($response['donors'] as $donor) {
        try {
          // Check if we already have a user for this donor.
          $existing_user = $this->userService->findUserByDonorId($donor['id']);
          
          if ($existing_user) {
            // Update existing user.
            $this->userService->updateUser($existing_user, $donor);
            $updated++;
          }
          else {
            // Create new user.
            $new_user = $this->userService->createUser($donor);
            if ($new_user) {
              $created++;
            }
            else {
              $errors++;
              $this->logger->error('Failed to create user for donor @id during scheduled sync', [
                '@id' => $donor['id'] ?? 'unknown',
              ]);
            }
          }
        }
        catch (\Exception $e) {
          $errors++;
          $this->logger->error('Error processing donor @id during scheduled sync: @error', [
            '@id' => $donor['id'] ?? 'unknown',
            '@error' => $e->getMessage(),
          ]);
        }
      }
      
      $this->logger->info('Processed page @page of donor sync: @created created, @updated updated, @errors errors.', [
        '@page' => $data['page'],
        '@created' => $created,
        '@updated' => $updated,
        '@errors' => $errors,
      ]);
    }
    catch (\Exception $e) {
      $this->logger->error('Error processing donor sync page @page: @error', [
        '@page' => $data['page'],
        '@error' => $e->getMessage(),
      ]);
    }
  }
}
