<?php

namespace Drupal\raisedonors\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\key\KeyRepositoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use GuzzleHttp\Exception\ConnectException;
use GuzzleHttp\Exception\ServerException;
use GuzzleHttp\Exception\ClientException;

/**
 * Provides a service for interacting with the RaiseDonors API.
 */
class RaisedonorsApiService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The key repository.
   *
   * @var \Drupal\key\KeyRepositoryInterface
   */
  protected $keyRepository;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The RaiseDonors API key ID.
   *
   * @var string
   */
  protected $apiKeyId;

  /**
   * The RaiseDonors user service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsDrupalUserService
   */
  protected $userService;

  /**
   * Constructs a new RaisedonorsApiService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\key\KeyRepositoryInterface $key_repository
   *   The key repository.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\raisedonors\Service\RaisedonorsDrupalUserService $user_service
   *   The RaiseDonors user service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    KeyRepositoryInterface $key_repository,
    LoggerChannelInterface $logger,
    ClientInterface $http_client,
    RaisedonorsDrupalUserService $user_service
  ) {
    $this->configFactory = $config_factory;
    $this->keyRepository = $key_repository;
    $this->logger = $logger;
    $this->httpClient = $http_client;
    $this->userService = $user_service;
    
    // Load configuration.
    $config = $this->configFactory->get('raisedonors.settings');
    $this->apiKeyId = $config->get('api_key');
  }

  /**
   * Set the API key ID.
   *
   * @param string $api_key_id
   *   The API key ID.
   */
  public function setApiKeyId(string $api_key_id): void {
    $this->apiKeyId = $api_key_id;
  }

  /**
   * Tests the connection to the RaiseDonors API.
   *
   * @return bool
   *   TRUE if the connection was successful, FALSE otherwise.
   */
  public function testConnection(): bool {
    try {
      // Use the Donor/list endpoint directly with minimal data retrieval.
      $response = $this->makeRequest('GET', '/Donor/list', [
        'Take' => 10,
        'SortBy' => 'Id',
        'Descending' => 'true',
      ]);
      return !empty($response);
    }
    catch (ConnectException $e) {
      $this->logger->error('Failed to connect to RaiseDonors API: Connection error - @error', [
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
    catch (ClientException $e) {
      $response = $e->getResponse();
      $status_code = $response ? $response->getStatusCode() : 'unknown';
      $body = $response ? $response->getBody()->getContents() : 'No response body';
      
      $this->logger->error('RaiseDonors API client error: HTTP @code - @error. Response: @body', [
        '@code' => $status_code,
        '@error' => $e->getMessage(),
        '@body' => $body,
      ]);
      return FALSE;
    }
    catch (ServerException $e) {
      $response = $e->getResponse();
      $status_code = $response ? $response->getStatusCode() : 'unknown';
      
      $this->logger->error('RaiseDonors API server error: HTTP @code - @error', [
        '@code' => $status_code,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to connect to RaiseDonors API: Unexpected error - @error (@class)', [
        '@error' => $e->getMessage(),
        '@class' => get_class($e),
      ]);
      return FALSE;
    }
  }

  /**
   * Syncs donors from RaiseDonors to Drupal.
   *
   * @param array $options
   *   Additional options for the sync.
   *
   * @return int
   *   The number of donors synced.
   */
  public function syncDonors(array $options = []): int {
    // Check if we're in test mode.
    $config = $this->configFactory->get('raisedonors.settings');
    if ($config->get('mode') === 'test') {
      $this->logger->notice('Skipping donor sync because RaiseDonors module is in test mode.');
      return 0;
    }
    
    try {
      $count = 0;
      $pageNumber = 1;
      $has_more = TRUE;
      
      while ($has_more) {
        // Using the RaiseDonors API pagination parameters with the correct endpoint.
        $response = $this->makeRequest('GET', '/Donor/list', [
          'Take' => 100,
          'Skip' => ($pageNumber - 1) * 100,
          'SortBy' => 'Id',
          'Descending' => 'true',
        ]);
        
        if (empty($response) || !isset($response['donors']) || empty($response['donors'])) {
          $this->logger->notice('No more donors found on page @page', ['@page' => $pageNumber]);
          $has_more = FALSE;
          continue;
        }
        
        $this->logger->info('Processing page @page with @count donors', [
          '@page' => $pageNumber,
          '@count' => count($response['donors']),
        ]);
        
        foreach ($response['donors'] as $donor) {
          $this->processDonor($donor);
          $count++;
        }
        
        // Check if there are more pages.
        $total_count = $response['total'] ?? 0;
        $has_more = $count < $total_count;
        $pageNumber++;
        
        // Log progress for long-running sync operations.
        if ($has_more && $pageNumber % 5 === 0) {
          $this->logger->info('Donor sync progress: @processed of @total donors processed (@percent%)', [
            '@processed' => $count,
            '@total' => $total_count,
            '@percent' => number_format(($count / $total_count) * 100, 1),
          ]);
        }
      }
      
      $this->logger->notice('Successfully synced @count donors from RaiseDonors.', ['@count' => $count]);
      return $count;
    }
    catch (ConnectException $e) {
      $this->logger->error('Failed to sync donors from RaiseDonors: Connection error - @error', [
        '@error' => $e->getMessage(),
      ]);
      return 0;
    }
    catch (ClientException $e) {
      $response = $e->getResponse();
      $status_code = $response ? $response->getStatusCode() : 'unknown';
      $body = $response ? $response->getBody()->getContents() : 'No response body';
      
      $this->logger->error('Failed to sync donors: Client error HTTP @code - @error. Response: @body', [
        '@code' => $status_code,
        '@error' => $e->getMessage(),
        '@body' => $body,
      ]);
      return 0;
    }
    catch (ServerException $e) {
      $response = $e->getResponse();
      $status_code = $response ? $response->getStatusCode() : 'unknown';
      
      $this->logger->error('Failed to sync donors: Server error HTTP @code - @error', [
        '@code' => $status_code,
        '@error' => $e->getMessage(),
      ]);
      return 0;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to sync donors from RaiseDonors: Unexpected error - @error (@class)', [
        '@error' => $e->getMessage(),
        '@class' => get_class($e),
      ]);
      return 0;
    }
  }

  /**
   * Process a single donor.
   *
   * @param array $donor
   *   The donor data from RaiseDonors.
   */
  protected function processDonor(array $donor): void {
    try {
      // Check if we already have a user for this donor.
      $existing_user = $this->userService->findUserByDonorId($donor['id']);
      
      if ($existing_user) {
        // Update existing user.
        $this->userService->updateUser($existing_user, $donor);
        $this->logger->info('Updated Drupal user for donor @id', ['@id' => $donor['id']]);
      }
      else {
        // Create new user.
        $new_user = $this->userService->createUser($donor);
        if ($new_user) {
          $this->logger->info('Created new Drupal user for donor @id', ['@id' => $donor['id']]);
        }
        else {
          $this->logger->error('Failed to create user for donor @id - user creation returned NULL', [
            '@id' => $donor['id'] ?? 'unknown',
          ]);
        }
      }
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to process donor @id: @error (@class) - Trace: @trace', [
        '@id' => $donor['id'] ?? 'unknown',
        '@error' => $e->getMessage(),
        '@class' => get_class($e),
        '@trace' => $e->getTraceAsString(),
      ]);
    }
  }

  /**
   * Makes a request to the RaiseDonors API.
   *
   * @param string $method
   *   The HTTP method.
   * @param string $endpoint
   *   The API endpoint.
   * @param array $params
   *   Query parameters or request body.
   * @param array $headers
   *   Additional headers.
   *
   * @return array|null
   *   The parsed response or NULL on failure.
   *
   * @throws \GuzzleHttp\Exception\RequestException
   *   Thrown on request failure.
   * @throws \Exception
   *   Thrown on API key issues or response parsing issues.
   */
  public function makeRequest(string $method, string $endpoint, array $params = [], array $headers = []): ?array {
    // Validate that we have the required API key.
    if (empty($this->apiKeyId)) {
      $this->logger->error('API request failed: Missing RaiseDonors API key ID');
      throw new \Exception('Missing RaiseDonors API key');
    }
    
    // Get the API key value.
    $key = $this->keyRepository->getKey($this->apiKeyId);
    if (!$key) {
      $this->logger->error('API request failed: API key with ID @id not found', [
        '@id' => $this->apiKeyId,
      ]);
      throw new \Exception('RaiseDonors API key not found: ' . $this->apiKeyId);
    }
    
    $api_key = $key->getKeyValue();
    if (empty($api_key)) {
      $this->logger->error('API request failed: API key with ID @id has no value', [
        '@id' => $this->apiKeyId,
      ]);
      throw new \Exception('Invalid RaiseDonors API key (empty value)');
    }
    
    // Prepare the request.
    $base_url = 'https://prod-api.raisedonors.com/api';
    $url = $base_url . $endpoint;
    
    $options = [
      'headers' => array_merge([
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $api_key,
      ], $headers),
    ];
    
    // Add parameters based on HTTP method.
    if ($method === 'GET') {
      $options['query'] = $params;
      $request_description = 'with query params: ' . json_encode($params);
    }
    else {
      $options['json'] = $params;
      $request_description = 'with body: ' . json_encode($params);
    }
    
    // Log the request (with sensitive data redacted).
    $this->logger->debug('Making @method request to @url @desc', [
      '@method' => $method,
      '@url' => $url,
      '@desc' => $request_description,
    ]);
    
    try {
      $start_time = microtime(TRUE);
      $response = $this->httpClient->request($method, $url, $options);
      $request_time = microtime(TRUE) - $start_time;
      
      $body = $response->getBody()->getContents();
      $status_code = $response->getStatusCode();
      
      // Log timing information for performance monitoring.
      $this->logger->debug('API request completed in @time seconds (HTTP @code)', [
        '@time' => number_format($request_time, 3),
        '@code' => $status_code,
      ]);
      
      // Parse the response.
      $data = json_decode($body, TRUE);
      if (json_last_error() !== JSON_ERROR_NONE) {
        $this->logger->error('Failed to parse API response: @error - Response: @body', [
          '@error' => json_last_error_msg(),
          '@body' => substr($body, 0, 1000) . (strlen($body) > 1000 ? '...' : ''),
        ]);
        throw new \Exception('Failed to parse RaiseDonors API response: ' . json_last_error_msg());
      }
      
      return $data;
    }
    catch (ClientException $e) {
      $response = $e->getResponse();
      $status_code = $response ? $response->getStatusCode() : 'unknown';
      $response_body = $response ? $response->getBody()->getContents() : 'No response body';
      
      // Log different information based on the status code.
      switch ($status_code) {
        case 401:
          $this->logger->error('Unauthorized API request (HTTP 401): Check your API key');
          break;
          
        case 403:
          $this->logger->error('Forbidden API request (HTTP 403): Permission denied');
          break;
          
        case 404:
          $this->logger->error('API endpoint not found (HTTP 404): Check endpoint @endpoint', [
            '@endpoint' => $endpoint,
          ]);
          break;
          
        default:
          $this->logger->error('API client error (HTTP @code): @error - Response: @body', [
            '@code' => $status_code,
            '@error' => $e->getMessage(),
            '@body' => $response_body,
          ]);
      }
      
      throw $e;
    }
    catch (ServerException $e) {
      $response = $e->getResponse();
      $status_code = $response ? $response->getStatusCode() : 'unknown';
      $response_body = $response ? $response->getBody()->getContents() : 'No response body';
      
      $this->logger->error('API server error (HTTP @code): @error - Response: @body', [
        '@code' => $status_code,
        '@error' => $e->getMessage(),
        '@body' => $response_body,
      ]);
      
      throw $e;
    }
    catch (ConnectException $e) {
      $this->logger->error('API connection error: @error - Timeout or DNS failure may have occurred', [
        '@error' => $e->getMessage(),
      ]);
      
      throw $e;
    }
    catch (RequestException $e) {
      $this->logger->error('API request error: @error', [
        '@error' => $e->getMessage(),
      ]);
      
      throw $e;
    }
  }
}
