<?php

namespace Drupal\raisedonors\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Password\PasswordGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\user\UserInterface;
use Drupal\Core\Entity\EntityStorageException;

/**
 * Provides a service for managing Drupal users from RaiseDonors data.
 */
class RaisedonorsDrupalUserService {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The password generator.
   *
   * @var \Drupal\Core\Password\PasswordGeneratorInterface
   */
  protected $passwordGenerator;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * Constructs a new RaisedonorsDrupalUserService object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Password\PasswordGeneratorInterface $password_generator
   *   The password generator.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger channel.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    PasswordGeneratorInterface $password_generator,
    LoggerChannelInterface $logger
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->passwordGenerator = $password_generator;
    $this->logger = $logger;
  }

  /**
   * Finds a user by donor ID.
   *
   * @param string $donor_id
   *   The RaiseDonors donor ID.
   *
   * @return \Drupal\user\UserInterface|null
   *   The user entity or NULL if not found.
   */
  public function findUserByDonorId(string $donor_id): ?\Drupal\user\UserInterface {
    try {
      $users = $this->entityTypeManager->getStorage('user')->loadByProperties([
        'donor_id' => $donor_id,
      ]);
      
      if (empty($users)) {
        $this->logger->debug('No user found with donor ID @id', ['@id' => $donor_id]);
        return NULL;
      }
      
      $user = reset($users);
      $this->logger->debug('Found user @name (uid: @uid) with donor ID @id', [
        '@name' => $user->getAccountName(),
        '@uid' => $user->id(),
        '@id' => $donor_id,
      ]);
      
      return $user;
    }
    catch (EntityStorageException $e) {
      $this->logger->error('Entity storage error while finding user by donor ID @id: @error', [
        '@id' => $donor_id,
        '@error' => $e->getMessage(),
      ]);
      return NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Unexpected error while finding user by donor ID @id: @error (@class)', [
        '@id' => $donor_id,
        '@error' => $e->getMessage(),
        '@class' => get_class($e),
      ]);
      return NULL;
    }
  }

  /**
   * Creates a new Drupal user from RaiseDonors donor data.
   *
   * @param array $donor
   *   The donor data from RaiseDonors.
   *
   * @return \Drupal\user\UserInterface|null
   *   The created user or NULL on failure.
   */
  public function createUser(array $donor): ?\Drupal\user\UserInterface {
    try {
      // Validate that we have all required data.
      if (empty($donor['Id'])) {
        $this->logger->error('Cannot create user: Donor ID is missing');
        return NULL;
      }
      
      // Extract primary email
      $email = $this->getPrimaryEmail($donor);
      
      if (empty($email)) {
        $this->logger->error('Cannot create user for donor @id: No email address found', [
          '@id' => $donor['Id'],
        ]);
        return NULL;
      }
      
      // Check if email is already in use.
      $existing_users = $this->entityTypeManager->getStorage('user')->loadByProperties([
        'mail' => $email,
      ]);
      
      if (!empty($existing_users)) {
        $existing_user = reset($existing_users);
        $this->logger->warning('Cannot create user for donor @id: Email @email is already in use by user @uid', [
          '@id' => $donor['Id'],
          '@email' => $email,
          '@uid' => $existing_user->id(),
        ]);
        return NULL;
      }
      
      // Generate a random password
      $password = $this->passwordGenerator->generate(10);
      
      // Fallback password generation if needed
      if (empty($password)) {
        $this->logger->notice('Password generator returned empty password, using fallback method');
        $password = $this->generateAlphanumericPassword();
      }
      
      // Create the user
      $username = $this->generateUsername($donor);
      
      $this->logger->debug('Creating new user with name @name and email @email for donor @id', [
        '@name' => $username,
        '@email' => $email,
        '@id' => $donor['Id'],
      ]);
      
      $user = $this->entityTypeManager->getStorage('user')->create([
        'name' => $username,
        'mail' => $email,
        'pass' => $password,
        'status' => 1,
      ]);
      
      // Set donor fields
      $this->setDonorFields($user, $donor);
      
      // Assign the donor role
      $user->addRole('donor');
      
      // Save the user
      $user->save();
      
      $this->logger->info('Created new user @name (uid: @uid) with Donor role for donor @id', [
        '@name' => $username,
        '@uid' => $user->id(),
        '@id' => $donor['Id'],
      ]);
      
      return $user;
    }
    catch (EntityStorageException $e) {
      $this->logger->error('Entity storage error creating user for donor @id: @error. Trace: @trace', [
        '@id' => $donor['Id'] ?? 'unknown',
        '@error' => $e->getMessage(),
        '@trace' => $e->getTraceAsString(),
      ]);
      return NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to create user from donor @id: @error (@class). Trace: @trace', [
        '@id' => $donor['Id'] ?? 'unknown',
        '@error' => $e->getMessage(),
        '@class' => get_class($e),
        '@trace' => $e->getTraceAsString(),
      ]);
      
      // Log the donor data for debugging (with sensitive info redacted).
      $donor_log = $donor;
      // Redact any sensitive data before logging.
      if (isset($donor_log['ContactMethods'])) {
        foreach ($donor_log['ContactMethods'] as &$method) {
          if (isset($method['Value']) && $method['IsEmail']) {
            // Partially redact email addresses.
            $parts = explode('@', $method['Value']);
            if (count($parts) === 2) {
              $method['Value'] = substr($parts[0], 0, 3) . '***@' . $parts[1];
            }
          }
        }
      }
      
      $this->logger->debug('Donor data that caused error: @data', [
        '@data' => json_encode($donor_log),
      ]);
      
      return NULL;
    }
  }

  /**
   * Updates an existing Drupal user with RaiseDonors donor data.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user to update.
   * @param array $donor
   *   The donor data from RaiseDonors.
   *
   * @return \Drupal\user\UserInterface|null
   *   The updated user or NULL on failure.
   */
  public function updateUser(\Drupal\user\UserInterface $user, array $donor): ?\Drupal\user\UserInterface {
    try {
      // Validate that we have all required data.
      if (empty($donor['Id'])) {
        $this->logger->error('Cannot update user: Donor ID is missing');
        return NULL;
      }
      
      $this->logger->debug('Updating user @name (uid: @uid) with donor @id data', [
        '@name' => $user->getAccountName(),
        '@uid' => $user->id(),
        '@id' => $donor['Id'],
      ]);
      
      // Update the email if changed
      $email = $this->getPrimaryEmail($donor);
      if ($email && $user->getEmail() !== $email) {
        // Check if this email is already in use by another user.
        $existing_users = $this->entityTypeManager->getStorage('user')->loadByProperties([
          'mail' => $email,
        ]);
        
        if (!empty($existing_users)) {
          foreach ($existing_users as $existing_user) {
            if ($existing_user->id() !== $user->id()) {
              $this->logger->warning('Cannot update user @uid email to @email: Email is already in use by user @existing_uid', [
                '@uid' => $user->id(),
                '@email' => $email,
                '@existing_uid' => $existing_user->id(),
              ]);
              // Don't update the email, but continue with other updates.
              $email = NULL;
              break;
            }
          }
        }
        
        if ($email) {
          $this->logger->info('Updating email for user @name (uid: @uid) from @old to @new', [
            '@name' => $user->getAccountName(),
            '@uid' => $user->id(),
            '@old' => $user->getEmail(),
            '@new' => $email,
          ]);
          
          $user->setEmail($email);
          
          // Only update username if it's based on email (matches the old email).
          if ($user->getAccountName() === $user->getEmail()) {
            $user->setUsername($email);
          }
        }
      }
      
      // Update donor fields
      $this->setDonorFields($user, $donor);
      
      // Ensure the user has the donor role
      if (!$user->hasRole('donor')) {
        $user->addRole('donor');
        $this->logger->info('Added Donor role to existing user @name (uid: @uid) for donor @id', [
          '@name' => $user->getAccountName(),
          '@uid' => $user->id(),
          '@id' => $donor['Id'],
        ]);
      }
      
      // Save the user
      $user->save();
      
      $this->logger->info('Successfully updated user @name (uid: @uid) with donor @id data', [
        '@name' => $user->getAccountName(),
        '@uid' => $user->id(),
        '@id' => $donor['Id'],
      ]);
      
      return $user;
    }
    catch (EntityStorageException $e) {
      $this->logger->error('Entity storage error updating user for donor @id: @error. Trace: @trace', [
        '@id' => $donor['Id'] ?? 'unknown',
        '@error' => $e->getMessage(),
        '@trace' => $e->getTraceAsString(),
      ]);
      return NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Failed to update user from donor @id: @error (@class). Trace: @trace', [
        '@id' => $donor['Id'] ?? 'unknown',
        '@error' => $e->getMessage(),
        '@class' => get_class($e),
        '@trace' => $e->getTraceAsString(),
      ]);
      return NULL;
    }
  }

  /**
   * Sets the donor fields on a user entity.
   *
   * @param \Drupal\user\UserInterface $user
   *   The user entity.
   * @param array $donor
   *   The donor data from RaiseDonors.
   *
   * @throws \Exception
   *   If there's an error setting donor fields.
   */
  protected function setDonorFields(\Drupal\user\UserInterface $user, array $donor): void {
    // Set donor ID
    if (isset($donor['Id'])) {
      $user->set('donor_id', $donor['Id']);
    }
    else {
      $this->logger->warning('Donor ID missing when setting donor fields for user @uid', [
        '@uid' => $user->id(),
      ]);
    }
    
    // Set name parsing
    $name_parts = [];
    if (isset($donor['Name'])) {
      // Log the name for debugging
      $this->logger->debug('Received donor name: @name', ['@name' => $donor['Name']]);
    }
    
    // Only set individual donor name fields if not an organization
    if (!isset($donor['IsOrganization']) || !$donor['IsOrganization']) {
      // Set first name
      if (isset($donor['FirstName'])) {
        $user->set('donor_first_name', $donor['FirstName']);
      }
      else {
        $this->logger->notice('First name missing in donor data for @id', [
          '@id' => $donor['Id'] ?? 'unknown',
        ]);
      }
      
      // Set middle name if present
      if (isset($donor['MiddleName'])) {
        $user->set('donor_middle_name', $donor['MiddleName']);
      }
      
      // Set last name
      if (isset($donor['LastName'])) {
        $user->set('donor_last_name', $donor['LastName']);
      }
      else {
        $this->logger->notice('Last name missing in donor data for @id', [
          '@id' => $donor['Id'] ?? 'unknown',
        ]);
      }
      
      // Set suffix if present
      if (isset($donor['Suffix'])) {
        $user->set('donor_suffix', $donor['Suffix']);
      }
      
      // Try to extract title from Name if it exists
      // Based on API response, title might be the first part of the name before the first name
      if (isset($donor['Name']) && isset($donor['FirstName']) && strpos($donor['Name'], $donor['FirstName']) !== false) {
        $name_prefix = trim(substr($donor['Name'], 0, strpos($donor['Name'], $donor['FirstName'])));
        if (!empty($name_prefix)) {
          $user->set('donor_title', $name_prefix);
          $this->logger->debug('Extracted donor title: @title', ['@title' => $name_prefix]);
        }
      }
    }
    else {
      $this->logger->debug('Donor @id is an organization (@name), skipping individual name fields', [
        '@id' => $donor['Id'] ?? 'unknown',
        '@name' => $donor['Name'] ?? 'Unknown Name',
      ]);
    }
    
    // Set email using only ContactMethods
    $email = $this->getPrimaryEmail($donor);
    if ($email) {
      $user->set('donor_email', $email);
    }
    else {
      $this->logger->notice('No primary email found in donor data for @id', [
        '@id' => $donor['Id'] ?? 'unknown',
      ]);
    }
    
    // Set phone from ContactMethods
    $phone = $this->getPrimaryPhone($donor);
    if ($phone) {
      $user->set('donor_phone', $phone);
    }
    
    // Set address
    $address = $this->getPrimaryAddress($donor);
    if ($address) {
      // If the donor is an organization, set the organization name in the address field
      if (isset($donor['IsOrganization']) && $donor['IsOrganization'] && isset($donor['Name'])) {
        $address['organization'] = $donor['Name'];
      }
      $user->set('donor_address', $address);
    }
    else {
      $this->logger->debug('No primary address found for donor @id', [
        '@id' => $donor['Id'] ?? 'unknown',
      ]);
    }
  }

  /**
   * Gets the primary email from donor data.
   *
   * @param array $donor
   *   The donor data.
   *
   * @return string|null
   *   The primary email or NULL if not found.
   */
  protected function getPrimaryEmail(array $donor): ?string {
    // Check in ContactMethods for email type and primary flag
    if (isset($donor['ContactMethods']) && is_array($donor['ContactMethods'])) {
      foreach ($donor['ContactMethods'] as $method) {
        // Check if contact method type is Email and Is Primary Email
        if (isset($method['IsEmail']) && $method['IsEmail'] && 
            isset($method['IsPrimary']) && $method['IsPrimary'] && 
            !empty($method['Value'])) {
          
          // Validate the email format
          if (filter_var($method['Value'], FILTER_VALIDATE_EMAIL)) {
            return $method['Value'];
          }
          else {
            $this->logger->warning('Found primary email for donor @id but it is not a valid email format: @email', [
              '@id' => $donor['Id'] ?? 'unknown',
              '@email' => $method['Value'],
            ]);
          }
        }
      }
      
      // If we didn't find a primary email, check for any email
      foreach ($donor['ContactMethods'] as $method) {
        if (isset($method['IsEmail']) && $method['IsEmail'] && !empty($method['Value'])) {
          if (filter_var($method['Value'], FILTER_VALIDATE_EMAIL)) {
            $this->logger->notice('No primary email found for donor @id, using first available email instead', [
              '@id' => $donor['Id'] ?? 'unknown',
            ]);
            return $method['Value'];
          }
        }
      }
    }
    
    return NULL;
  }

  /**
   * Gets the primary phone from donor data.
   *
   * @param array $donor
   *   The donor data.
   *
   * @return string|null
   *   The primary phone or NULL if not found.
   */
  protected function getPrimaryPhone(array $donor): ?string {
    // Check in ContactMethods (webhook format)
    if (isset($donor['ContactMethods']) && is_array($donor['ContactMethods'])) {
      foreach ($donor['ContactMethods'] as $method) {
        // Check if contact method type is Phone and Is Primary Phone
        if (isset($method['IsPhone']) && $method['IsPhone'] && 
            isset($method['IsPrimary']) && $method['IsPrimary'] && 
            isset($method['Value'])) {
          $this->logger->debug('Found primary phone from ContactMethods: @phone', ['@phone' => $method['Value']]);
          return $method['Value'];
        }
      }
      
      // If no primary phone found, use the first available phone
      foreach ($donor['ContactMethods'] as $method) {
        if (isset($method['IsPhone']) && $method['IsPhone'] && isset($method['Value'])) {
          $this->logger->debug('No primary phone found, using first available phone: @phone', [
            '@phone' => $method['Value'],
          ]);
          return $method['Value'];
        }
      }
    }
    
    return NULL;
  }

  /**
   * Gets the most recent address from donor data based on the highest ID value.
   *
   * @param array $donor
   *   The donor data.
   *
   * @return array|null
   *   The most recent address or NULL if not found.
   */
  protected function getPrimaryAddress(array $donor): ?array {
    // Check in Address array (webhook format)
    if (isset($donor['Address']) && is_array($donor['Address'])) {
      // Find the address with the highest ID value
      $most_recent_address = null;
      $highest_id = 0;
      
      foreach ($donor['Address'] as $address) {
        if (isset($address['Id']) && $address['Id'] > $highest_id) {
          $highest_id = $address['Id'];
          $most_recent_address = $address;
        }
      }
      
      if ($most_recent_address) {
        $this->logger->debug('Found most recent address (ID: @id) for donor @donor_id', [
          '@id' => $highest_id,
          '@donor_id' => $donor['Id'] ?? 'unknown',
        ]);
        
        return [
          'country_code' => $most_recent_address['Country'] ?? '',
          'address_line1' => $most_recent_address['Address1'] ?? '',
          'address_line2' => $most_recent_address['Address2'] ?? '',
          'postal_code' => $most_recent_address['PostalCode'] ?? '',
          'locality' => $most_recent_address['City'] ?? '',
          'administrative_area' => $most_recent_address['State'] ?? '',
          'organization' => '',
        ];
      }
    }
    
    return NULL;
  }

  /**
   * Generates a username based on donor data.
   *
   * @param array $donor
   *   The donor data.
   *
   * @return string
   *   The generated username.
   */
  protected function generateUsername(array $donor): string {
    // First try to use the email address as the username.
    $email = $this->getPrimaryEmail($donor);
    if (!empty($email)) {
      // Check if this username already exists.
      $existing = $this->entityTypeManager->getStorage('user')->loadByProperties([
        'name' => $email,
      ]);
      
      if (empty($existing)) {
        return $email;
      }
      else {
        $this->logger->debug('Username @name already exists, will generate alternative', [
          '@name' => $email,
        ]);
      }
    }
    
    // If email is unavailable or already taken, try name-based username.
    $base_name = '';
    
    if (isset($donor['FirstName']) && isset($donor['LastName'])) {
      // Use first and last name combo.
      $base_name = strtolower($donor['FirstName'] . '.' . $donor['LastName']);
      
      // Remove any special characters.
      $base_name = preg_replace('/[^a-z0-9\.]/', '', $base_name);
    }
    elseif (isset($donor['Name'])) {
      // Use the full name, converted to a username format.
      $base_name = strtolower(str_replace(' ', '.', $donor['Name']));
      
      // Remove any special characters.
      $base_name = preg_replace('/[^a-z0-9\.]/', '', $base_name);
    }
    else {
      // If no name information, use a generic username with the donor ID.
      $base_name = 'donor.' . $donor['Id'];
    }
    
    // Check if this username already exists.
    $username = $base_name;
    $i = 1;
    
    while (TRUE) {
      $existing = $this->entityTypeManager->getStorage('user')->loadByProperties([
        'name' => $username,
      ]);
      
      if (empty($existing)) {
        break;
      }
      
      // Username exists, try adding a number.
      $username = $base_name . $i;
      $i++;
      
      // Safety check to avoid infinite loop.
      if ($i > 100) {
        // If we can't find a unique username after 100 attempts, use a random one.
        $username = 'donor_' . $donor['Id'] . '_' . substr(md5(microtime()), 0, 8);
        $this->logger->warning('Could not generate unique username after 100 attempts, using random username @name', [
          '@name' => $username,
        ]);
        break;
      }
    }
    
    $this->logger->debug('Generated username @name for donor @id', [
      '@name' => $username,
      '@id' => $donor['Id'] ?? 'unknown',
    ]);
    
    return $username;
  }

  /**
   * Fallback method to generate an alphanumeric password.
   *
   * @return string
   *   A 12-character alphanumeric password with special characters.
   */
  protected function generateAlphanumericPassword(): string {
    $length = 12;
    $chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!@#$%^&*()-_=+';
    $password = '';
    
    try {
      // Use random_int for cryptographically secure random numbers.
      for ($i = 0; $i < $length; $i++) {
        $password .= $chars[random_int(0, strlen($chars) - 1)];
      }
      
      $this->logger->debug('Generated fallback password successfully');
    }
    catch (\Exception $e) {
      // Fallback to less secure but still usable method if random_int fails.
      $this->logger->warning('Failed to use random_int for password generation: @error. Using less secure fallback.', [
        '@error' => $e->getMessage(),
      ]);
      
      $chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
      for ($i = 0; $i < $length; $i++) {
        $password .= $chars[mt_rand(0, strlen($chars) - 1)];
      }
    }
    
    return $password;
  }
}
