<?php

namespace Drupal\Tests\raisedonors\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\raisedonors\Service\RaisedonorsApiService;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\key\KeyRepositoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Psr7\Response;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * Tests the RaiseDonors API service.
 *
 * @group raisedonors
 */
class RaisedonorsApiServiceTest extends KernelTestBase {

  use ProphecyTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'user',
    'key',
    'address',
    'raisedonors',
  ];

  /**
   * The RaiseDonors API service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsApiService
   */
  protected $apiService;

  /**
   * The config factory prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $configFactory;

  /**
   * The key repository prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $keyRepository;

  /**
   * The logger channel prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $logger;

  /**
   * The HTTP client prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['system', 'user']);
    $this->installEntitySchema('user');

    // Set up prophecies.
    $this->configFactory = $this->prophesize(ConfigFactoryInterface::class);
    $this->keyRepository = $this->prophesize(KeyRepositoryInterface::class);
    $this->logger = $this->prophesize(LoggerChannelInterface::class);
    $this->httpClient = $this->prophesize(ClientInterface::class);
    $userService = $this->prophesize(\Drupal\raisedonors\Service\RaisedonorsDrupalUserService::class);

    // Create the API service with mocked dependencies.
    $this->apiService = new RaisedonorsApiService(
      $this->configFactory->reveal(),
      $this->keyRepository->reveal(),
      $this->logger->reveal(),
      $this->httpClient->reveal(),
      $userService->reveal()
    );
  }

  /**
   * Tests the setApiKeyId method.
   */
  public function testSetApiKeyId(): void {
    $api_key_id = 'test_api_key';

    $this->apiService->setApiKeyId($api_key_id);

    // Use reflection to access protected properties.
    $reflection = new \ReflectionClass($this->apiService);
    
    $api_key_property = $reflection->getProperty('apiKeyId');
    $api_key_property->setAccessible(TRUE);

    $this->assertEquals($api_key_id, $api_key_property->getValue($this->apiService));
  }

  /**
   * Tests the makeRequest method.
   */
  public function testMakeRequest(): void {
    $api_key_id = 'test_api_key';
    $api_key_value = 'test_key_value';
    $test_response = [
      'total' => 2,
      'donors' => [
        [
          'id' => '123',
          'FirstName' => 'John',
          'LastName' => 'Doe',
          'ContactMethods' => [
            [
              'IsEmail' => TRUE,
              'IsPrimary' => TRUE,
              'Value' => 'john.doe@example.com',
            ],
          ],
        ],
        [
          'id' => '456',
          'FirstName' => 'Jane',
          'LastName' => 'Smith',
          'ContactMethods' => [
            [
              'IsEmail' => TRUE,
              'IsPrimary' => TRUE,
              'Value' => 'jane.smith@example.com',
            ],
          ],
        ],
      ],
    ];
    
    // Mock the key repository to return our test key.
    $key = $this->prophesize(\Drupal\key\KeyInterface::class);
    $key->getKeyValue()->willReturn($api_key_value);
    $this->keyRepository->getKey($api_key_id)->willReturn($key->reveal());
    
    // Mock HTTP client response.
    $response_mock = new Response(
      200,
      ['Content-Type' => 'application/json'],
      json_encode($test_response)
    );
    
    // Expect a request with Bearer token authentication to the correct endpoint.
    $this->httpClient->request(
      'GET',
      'https://prod-api.raisedonors.com/api/Donor/list',
      \Prophecy\Argument::that(function ($options) use ($api_key_value) {
        return isset($options['headers']['Authorization']) 
          && $options['headers']['Authorization'] === 'Bearer ' . $api_key_value
          && isset($options['query']['Take']);
      })
    )->willReturn($response_mock);
    
    // Set up the API service.
    $this->apiService->setApiKeyId($api_key_id);
    
    // Call makeRequest and verify response.
    $result = $this->apiService->makeRequest('GET', '/Donor/list', ['Take' => 10]);
    $this->assertEquals($test_response, $result);
  }

  /**
   * Tests the testConnection method.
   */
  public function testTestConnection(): void {
    $api_key_id = 'test_api_key';
    $api_key_value = 'test_key_value';
    $test_response = [
      'total' => 0,
      'donors' => [],
    ];
    
    // Mock the key repository to return our test key.
    $key = $this->prophesize(\Drupal\key\KeyInterface::class);
    $key->getKeyValue()->willReturn($api_key_value);
    $this->keyRepository->getKey($api_key_id)->willReturn($key->reveal());
    
    // Mock HTTP client response.
    $response_mock = new Response(
      200,
      ['Content-Type' => 'application/json'],
      json_encode($test_response)
    );
    
    // Expect a request with correct parameters for testConnection.
    $this->httpClient->request(
      'GET',
      'https://prod-api.raisedonors.com/api/Donor/list',
      \Prophecy\Argument::that(function ($options) use ($api_key_value) {
        return isset($options['headers']['Authorization']) 
          && $options['headers']['Authorization'] === 'Bearer ' . $api_key_value
          && isset($options['query']['Take'])
          && $options['query']['Take'] === 10
          && isset($options['query']['SortBy'])
          && $options['query']['SortBy'] === 'Id'
          && isset($options['query']['Descending'])
          && $options['query']['Descending'] === 'true';
      })
    )->willReturn($response_mock);
    
    // Set up the API service.
    $this->apiService->setApiKeyId($api_key_id);
    
    // Call testConnection and verify it returns true for a valid response.
    $result = $this->apiService->testConnection();
    $this->assertTrue($result);
  }

  /**
   * Tests the syncDonors method.
   */
  public function testSyncDonors(): void {
    $api_key_id = 'test_api_key';
    $api_key_value = 'test_key_value';
    
    // Create test config for mode.
    $config = $this->prophesize(\Drupal\Core\Config\ImmutableConfig::class);
    $config->get('mode')->willReturn('live');
    $this->configFactory->get('raisedonors.settings')->willReturn($config->reveal());
    
    // Mock the key repository.
    $key = $this->prophesize(\Drupal\key\KeyInterface::class);
    $key->getKeyValue()->willReturn($api_key_value);
    $this->keyRepository->getKey($api_key_id)->willReturn($key->reveal());
    
    // Create test response for donor list, page 1.
    $test_response_page1 = [
      'total' => 2,
      'donors' => [
        [
          'id' => '123',
          'FirstName' => 'John',
          'LastName' => 'Doe',
          'ContactMethods' => [
            [
              'IsEmail' => TRUE,
              'IsPrimary' => TRUE,
              'Value' => 'john.doe@example.com',
            ],
          ],
        ],
      ],
    ];
    
    // Create test response for donor list, page 2.
    $test_response_page2 = [
      'total' => 2,
      'donors' => [
        [
          'id' => '456',
          'FirstName' => 'Jane',
          'LastName' => 'Smith',
          'ContactMethods' => [
            [
              'IsEmail' => TRUE,
              'IsPrimary' => TRUE,
              'Value' => 'jane.smith@example.com',
            ],
          ],
        ],
      ],
    ];
    
    // Create test response for donor list, page 3 (empty).
    $test_response_page3 = [
      'total' => 2,
      'donors' => [],
    ];
    
    // Mock HTTP client responses for each page.
    $response_mock_page1 = new Response(
      200,
      ['Content-Type' => 'application/json'],
      json_encode($test_response_page1)
    );
    
    $response_mock_page2 = new Response(
      200,
      ['Content-Type' => 'application/json'],
      json_encode($test_response_page2)
    );
    
    $response_mock_page3 = new Response(
      200,
      ['Content-Type' => 'application/json'],
      json_encode($test_response_page3)
    );
    
    // Set up HTTP client to return different responses for different pages.
    $this->httpClient
      ->request(
        'GET',
        'https://prod-api.raisedonors.com/api/Donor/list',
        \Prophecy\Argument::that(function ($options) {
          return $options['query']['Skip'] === 0;
        })
      )
      ->willReturn($response_mock_page1);
    
    $this->httpClient
      ->request(
        'GET',
        'https://prod-api.raisedonors.com/api/Donor/list',
        \Prophecy\Argument::that(function ($options) {
          return $options['query']['Skip'] === 100;
        })
      )
      ->willReturn($response_mock_page2);
    
    $this->httpClient
      ->request(
        'GET',
        'https://prod-api.raisedonors.com/api/Donor/list',
        \Prophecy\Argument::that(function ($options) {
          return $options['query']['Skip'] === 200;
        })
      )
      ->willReturn($response_mock_page3);
    
    // Set up the API service.
    $this->apiService->setApiKeyId($api_key_id);
    
    // Call syncDonors and verify it returns the correct count.
    $result = $this->apiService->syncDonors();
    $this->assertEquals(2, $result);
  }

}
