<?php

namespace Drupal\Tests\raisedonors\Unit;

use Drupal\Tests\UnitTestCase;
use Drupal\raisedonors\Service\RaisedonorsDrupalUserService;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Password\PasswordGeneratorInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Prophecy\PhpUnit\ProphecyTrait;

/**
 * Tests the RaiseDonors Drupal User Service.
 *
 * @group raisedonors
 * @coversDefaultClass \Drupal\raisedonors\Service\RaisedonorsDrupalUserService
 */
class RaisedonorsDrupalUserServiceTest extends UnitTestCase {

  use ProphecyTrait;

  /**
   * The user service.
   *
   * @var \Drupal\raisedonors\Service\RaisedonorsDrupalUserService
   */
  protected $userService;

  /**
   * The entity type manager prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $entityTypeManager;

  /**
   * The password generator prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $passwordGenerator;

  /**
   * The logger channel prophecy.
   *
   * @var \Prophecy\Prophecy\ObjectProphecy
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Set up prophecies.
    $this->entityTypeManager = $this->prophesize(EntityTypeManagerInterface::class);
    $this->passwordGenerator = $this->prophesize(PasswordGeneratorInterface::class);
    $this->logger = $this->prophesize(LoggerChannelInterface::class);

    // Create the user service with mocked dependencies.
    $this->userService = new RaisedonorsDrupalUserService(
      $this->entityTypeManager->reveal(),
      $this->passwordGenerator->reveal(),
      $this->logger->reveal()
    );
  }

  /**
   * Tests the username generation with email.
   *
   * @covers ::generateUsername
   */
  public function testGenerateUsernameWithEmail(): void {
    // Create donor data structure matching the implementation expectations
    $donor = [
      'Id' => '12345',
      'FirstName' => 'John',
      'LastName' => 'Doe',
      'ContactMethods' => [
        [
          'IsEmail' => TRUE,
          'IsPrimary' => TRUE,
          'Value' => 'john.doe@example.com',
        ],
      ],
    ];

    // Mock the user storage to simulate no existing users.
    $user_storage = $this->prophesize('Drupal\Core\Entity\EntityStorageInterface');
    $user_storage->loadByProperties(['name' => 'john.doe@example.com'])->willReturn([]);
    $this->entityTypeManager->getStorage('user')->willReturn($user_storage->reveal());

    // Use reflection to access the protected method.
    $reflection = new \ReflectionClass($this->userService);
    $method = $reflection->getMethod('generateUsername');
    $method->setAccessible(TRUE);

    // Call the method and verify result.
    $result = $method->invoke($this->userService, $donor);
    $this->assertEquals('john.doe@example.com', $result);
  }

  /**
   * Tests primary email extraction from donor data.
   *
   * @covers ::getPrimaryEmail
   */
  public function testGetPrimaryEmail(): void {
    // Create donor data structure matching the implementation expectations
    $donor = [
      'Id' => '12345',
      'ContactMethods' => [
        [
          'IsEmail' => TRUE,
          'IsPrimary' => FALSE,
          'Value' => 'secondary@example.com',
        ],
        [
          'IsEmail' => TRUE,
          'IsPrimary' => TRUE,
          'Value' => 'primary@example.com',
        ],
      ],
    ];

    // Use reflection to access the protected method.
    $reflection = new \ReflectionClass($this->userService);
    $method = $reflection->getMethod('getPrimaryEmail');
    $method->setAccessible(TRUE);

    // Call the method and verify it returns the primary email.
    $result = $method->invoke($this->userService, $donor);
    $this->assertEquals('primary@example.com', $result);
  }

  /**
   * Tests primary address extraction from donor data.
   *
   * @covers ::getPrimaryAddress
   */
  public function testGetPrimaryAddress(): void {
    // Create donor data structure matching the implementation expectations
    $donor = [
      'Id' => '12345',
      'Address' => [
        [
          'Id' => 1,
          'Country' => 'US',
          'Address1' => '123 Secondary St',
          'City' => 'Anytown',
          'State' => 'NY',
          'PostalCode' => '12345',
        ],
        [
          'Id' => 2,
          'Country' => 'US',
          'Address1' => '456 Main St',
          'Address2' => 'Apt 789',
          'City' => 'Somewhere',
          'State' => 'CA',
          'PostalCode' => '90210',
        ],
      ],
    ];

    // Use reflection to access the protected method.
    $reflection = new \ReflectionClass($this->userService);
    $method = $reflection->getMethod('getPrimaryAddress');
    $method->setAccessible(TRUE);

    // Call the method and verify it returns the correct address data.
    $result = $method->invoke($this->userService, $donor);
    $expected = [
      'country_code' => 'US',
      'address_line1' => '456 Main St',
      'address_line2' => 'Apt 789',
      'locality' => 'Somewhere',
      'administrative_area' => 'CA',
      'postal_code' => '90210',
      'organization' => '',
    ];
    $this->assertEquals($expected, $result);
  }

  /**
   * Tests primary phone extraction from donor data.
   *
   * @covers ::getPrimaryPhone
   */
  public function testGetPrimaryPhone(): void {
    // Create donor data structure matching the implementation expectations
    $donor = [
      'Id' => '12345',
      'ContactMethods' => [
        [
          'IsPhone' => TRUE,
          'IsPrimary' => FALSE,
          'Value' => '555-123-4567',
        ],
        [
          'IsPhone' => TRUE,
          'IsPrimary' => TRUE,
          'Value' => '555-987-6543',
        ],
      ],
    ];

    // Use reflection to access the protected method.
    $reflection = new \ReflectionClass($this->userService);
    $method = $reflection->getMethod('getPrimaryPhone');
    $method->setAccessible(TRUE);

    // Call the method and verify it returns the primary phone.
    $result = $method->invoke($this->userService, $donor);
    $this->assertEquals('555-987-6543', $result);
  }

  /**
   * Tests fallback to non-primary email when no primary exists.
   *
   * @covers ::getPrimaryEmail
   */
  public function testFallbackToNonPrimaryEmail(): void {
    // Create donor data with no primary email
    $donor = [
      'Id' => '12345',
      'ContactMethods' => [
        [
          'IsEmail' => TRUE,
          'IsPrimary' => FALSE,
          'Value' => 'available@example.com',
        ],
      ],
    ];

    // Use reflection to access the protected method.
    $reflection = new \ReflectionClass($this->userService);
    $method = $reflection->getMethod('getPrimaryEmail');
    $method->setAccessible(TRUE);

    // Call the method and verify it returns the first available email.
    $result = $method->invoke($this->userService, $donor);
    $this->assertEquals('available@example.com', $result);
  }

  /**
   * Tests case when no email is found.
   *
   * @covers ::getPrimaryEmail
   */
  public function testNoEmailFound(): void {
    // Create donor with no emails
    $donor = [
      'Id' => '12345',
      'ContactMethods' => [
        [
          'IsPhone' => TRUE,
          'IsPrimary' => TRUE,
          'Value' => '555-987-6543',
        ],
      ],
    ];

    // Use reflection to access the protected method.
    $reflection = new \ReflectionClass($this->userService);
    $method = $reflection->getMethod('getPrimaryEmail');
    $method->setAccessible(TRUE);

    // Call the method and verify it returns null.
    $result = $method->invoke($this->userService, $donor);
    $this->assertNull($result);
  }

}
